# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import MessageV2
from driverng.splitted import Date, Splitted, String


class VmlinkBase(Splitted):
    SEPARATOR = ";"


class VmlinkResult(VmlinkBase):
    FIELDS = (
        String("id_instrument", 7),
        String("analyse", 10),
        String("id_echantillon", 16),
        String("resultat", 12),
        Date("date", 14, fmt_date="%Y%m%d%H%M%S"),
        # Flag alarme si connu
        String("flag", 10),
        # Lot du réactif si connu
        String("id_reactif", 20),
        # Lot du QC si connu
        String("id_lot_cq", 20),
    )


class VmlinkMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.ORDER: VmlinkResult}

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        for line in data.splitlines():
            if not len(line):
                continue
            type_message = MessageV2.ORDER
            obj = self.add(type_message=type_message)
            obj.parse(line)

    def to_string(self):
        return "\r\n".join([e.to_string() for e in self]) + "\r\n"


class Driver(MessageV2):
    SPEC = {"code": "vmlink", "label": "VM-LINK Validation Méthode"}

    def create(self):
        return VmlinkMultiple()

    def parse(self, data):
        obj = VmlinkMultiple()
        obj.parse(data)
        return obj

    def to_string(self, message):
        return message.to_string()

    def encode_worklist(self, worklist, context):
        message = self.create()
        for sample in worklist.samples:
            for result in sample.results:
                reagent = ""
                for r in result.lots:
                    if r.type == "reagent":
                        reagent = r.lot
                message.add(
                    type_message=MessageV2.ORDER,
                    id_instrument=result.analyseur
                    if result.get("analyseur")
                    else result.instrument,
                    analyse=result.code,
                    id_echantillon=sample.sample_id
                    if sample.type == "patient"
                    else sample.control_lot,
                    resultat=result.value,
                    date=result.date_instrument
                    if result.get("date_instrument")
                    else result.date,
                    flag=" ".join(result.flags),
                    id_reactif=reagent,
                )
        if len(message):
            return message
        else:
            return None
